<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * 認証プラグイン
 * 
 * @author ishinao <ishinao@ishinao.net>
 * $Id: AuthPlugin.php 72 2006-05-15 09:19:25Z ishinao $
 */

/**
 * プラグイン基底クラスの読み込み
 */
Zend::loadClass('Zend_Controller_Plugin_Abstract');

/**
 * 認証プラグインクラス
 */
class AuthPlugin extends Zend_Controller_Plugin_Abstract
{
	/**
	 * 認証情報ファイルパス
	 * 
	 * 認証情報ファイルには、改行区切りで1行目にID、2行目にパスワードを記述
	 */
	protected $_authFile = null;

	/**
	 * コンストラクタ
	 * 
	 * @param string $authFile 認証情報が記述されたファイル
	 */
	public function __construct($authFile)
	{

		$this->_authFile = $authFile; // 認証ファイルパスをセット

		// ファイルの存在確認
		if (!is_readable($this->_authFile)) {
			throw new Exception('AUTH FILE not readable: ' . $this->_authFile);
		}

	}

	/**
	 * ルーティング終了時に実行されるメソッド
	 * 
	 * 認証はプラグイン実行タイミングの最初の方で実行する必要がある。
	 * もっとも最初に実行されるメソッドはrouteStartupだが、routeStartup
	 * ではDispatcherTokenの差し替えができないので、その次のrouteShutdown
	 * で実行している。
	 */
	public function routeShutdown($action)
	{

		if (!session_id()) {session_start();} // セッションが実行されていなければスタート

		if (!$this->_isLogin()) {

			// 未ログインの場合はログインアクションにTokenを差し替え
			$action = new Zend_Controller_Dispatcher_Token('system', 'login');

		}

		return $action; // Tokenを返す

	}
	
	/**
	 * ログインチェックメソッド
	 */
	protected function _isLogin()
	{

		if ($_SESSION['login'] === true) {return true;} // すでにログイン済み

		$post = Zend::registry('post'); // $_POST入力フィルタ

		$lines = explode("\n", file_get_contents($this->_authFile)); // 認証情報ファイルを読み込み

		$inputId = $post->getAlpha('loginid'); // 入力からIDを取得。IDはアルファベットのみ許可
		$id = trim(array_shift($lines)); // ファイルから取得したID

		$inputPassword = $post->getRaw('password'); // 入力からパスワードを取得
		$password = trim(array_shift($lines)); // ファイルから取得したパスワード

		// ここでは平文パスワードを比較しているが、通常はcryptなどを使うべき
		if (($id == $inputId) && ($password == $inputPassword)) {

			// ID、パスワードが正しければセッションに認証状態を登録
			$_SESSION['login'] = true;
			return true; // 認証成功

		}

		return false; // 認証失敗

	}
	
	/**
	 * ログアウトメソッド
	 */
	public static function logout()
	{

		unset($_SESSION['login']); // セッションの認証状態を削除

	}
}

?>
